// COPYRIGHT (c) 2002 PLOTNIX, INC. All rights reserved

package com.plotnix.jex.adapters;

import com.plotnix.jex.Interpreter;
import com.plotnix.jex.Compiler;
import com.plotnix.jex.Expression;
import com.plotnix.jex.IterationExpression;
import com.plotnix.jex.VariableExpression;

/**
 * A wrapper for an interpreter that makes it behave like a compiler.
 * Each compilation method checks the syntax of the expression and
 * wraps the expression  into an {@link InterpretedExpression
 * InterpretedExpression}, which is a wrapper for the expression string.
 *
 * @author Dmitri Plotnikov
 * @version $Revision$
 */
public class InterpretingCompiler extends Compiler {

    private Interpreter interpreter;

    /**
     * @param interpreter is the interpreter that needs to be wrapped
     */    
    public InterpretingCompiler(Interpreter interpreter){
        this.interpreter = interpreter;
    }
    
    /**
     * @see Compiler#compileExpression(String)
     */
    public Expression compileExpression(String expression) {
        interpreter.validateExpression(expression);
        return new InterpretedExpression(interpreter, expression);
    }

    /**
     * @see Compiler#compileIterationExpression(String)
     */
    public IterationExpression compileIterationExpression(String expression) {
        interpreter.validateIterationExpression(expression);
        return new InterpretedExpression(interpreter, expression);
    }

    /**
     * @see Compiler#compileVariableExpression(String)
     */
    public VariableExpression compileVariableExpression(String expression) {
        interpreter.validateVariableExpression(expression);
        return new InterpretedExpression(interpreter, expression);
    }

}
