// COPYRIGHT (c) 2002 PLOTNIX, INC. All rights reserved

package com.plotnix.jex.jxpath;

import org.apache.commons.jxpath.CompiledExpression;
import org.apache.commons.jxpath.JXPathContext;

import com.plotnix.jex.Expression;
import com.plotnix.jex.IterationExpression;
import com.plotnix.jex.VariableExpression;

/**
 * Integrates JXPath with Jex. 
 * See <a href="http://jakarta.apache.org/commons/jxpath">
 * JXPath home page on Apache</a>.
 * <p>
 * If Jex becomes a part of Jakarta, JXPath will change to implement
 * Jex interfaces directly.  At that point the need for this adaptor
 * will disappear.
 * 
 * @author Dmitri Plotnikov
 * @version $Revision$ $Date$
 */
public class Compiler extends com.plotnix.jex.Compiler {

    private static final JXPathContext COMPILATION_CONTEXT =
        JXPathContext.newContext(null);
    
    
    /**
     * @see com.plotnix.jex.Compiler#compileExpression(String)
     */
    public Expression compileExpression(String expression) {
        CompiledExpression expr = COMPILATION_CONTEXT.compile(expression);
        return new JXPathCompiledExpression(expr);
    }

    /**
     * @see com.plotnix.jex.Compiler#compileIterationExpression(String)
     */
    public IterationExpression compileIterationExpression(String expression) {
        CompiledExpression expr = COMPILATION_CONTEXT.compile(expression);
        return new JXPathCompiledExpression(expr);
    }

    /**
     * @see com.plotnix.jex.Compiler#compileVariableExpression(String)
     */
    public VariableExpression compileVariableExpression(String expression) {
        CompiledExpression expr = COMPILATION_CONTEXT.compile(expression);
        return new JXPathCompiledExpression(expr);
    }

}
