// COPYRIGHT (c) 2002 PLOTNIX, INC. All rights reserved

package com.plotnix.jex.jxpath;

import java.util.Iterator;

import org.apache.commons.jxpath.CompiledExpression;
import org.apache.commons.jxpath.JXPathContext;

import com.plotnix.jex.Context;
import com.plotnix.jex.Expression;
import com.plotnix.jex.IterationExpression;
import com.plotnix.jex.VariableExpression;

/**
 * A trivial wrapper for JXPath CompiledExpression.
 * <p>
 * If Jex becomes a part of Jakarta, JXPath will change to implement
 * Jex interfaces directly.  At that point the need for this adaptor
 * will disappear.
 * 
 * @author Dmitri Plotnikov
 * @version $Revision$ $Date$
 */
public class JXPathCompiledExpression
    implements Expression, IterationExpression, VariableExpression {

    private static final String JXPATH_CONTEXT_PROPERTY = "jxpath.CONTEXT";
    
    private CompiledExpression expression;
    
    /**
     * Wraps a jxpath CompiledExpression into a Jex 
     * Expression/IterationExpression/VariableExpression
     * 
     * @param expression is a jxpath-style Compiled expression
     */
    public JXPathCompiledExpression(CompiledExpression expression){
        this.expression = expression;
    }
    
    /**
     * @see Expression#evaluate(Context, Object[])
     */
    public Object evaluate(Context context, Object[] arguments) {        
        return expression.getValue(getContext(context, arguments));
    }

    /**
     * @see Expression#evaluate(Context, Object[], Class)
     */
    public Object evaluate(Context context, 
                Object[] arguments, Class valueType) {
        return expression.getValue(getContext(context, arguments), valueType);
    }

    /**
     * @see IterationExpression#iterate(Context, Object[])
     */
    public Iterator iterate(Context context, Object[] arguments) {
        return expression.iterate(getContext(context, arguments));
    }

    /**
     * @see VariableExpression#assign(Context, Object, Object[])
     */
    public void assign(Context context, Object value, Object[] arguments) {
        expression.setValue(getContext(context, arguments), value);
    }

    /**
     * Creates a new JXPathContext for the evaluation of an expression. 
     * Makes the new context a child of the JXPathContext associated 
     * with the Context.
     * 
     * @see #getParentContext
     * 
     * @param context is a Jex Context
     * @param arguments should have the length of zero or one. The
     *    first argument of the array (or null) is used as the context bean.
     */
    private JXPathContext getContext(Context context, Object[] arguments){
        Object contextNode;
        if (arguments == null || arguments.length == 0){
            contextNode = null;
        }
        else if (arguments.length == 1){
            contextNode = arguments[0];
        }
        else {
            throw new RuntimeException("Too many arguments");
        }
        
        return JXPathContext.newContext(getParentContext(context), contextNode);
    }
    
    /**
     * Finds a JXPathContext associated with a Jex Context.
     * Creates a JXPathContext for each Jex Context and stores a pointer
     * to it in a Context's property.
     * 
     * @param context is a Jex Context
     * @param arguments should have the length of zero or one. The
     *    first argument of the array (or null) is used as the context bean.
     */
    private JXPathContext getParentContext(Context context){
        if (context == null){
            return null;
        }
        
        JXPathContext ctx = (JXPathContext) context.
                getProperty(JXPATH_CONTEXT_PROPERTY);
        if (ctx == null){
            JXPathContext parent = getParentContext(context.getParentContext());
            ctx = JXPathContext.newContext(parent, null);    
            ctx.setVariables(new JXPathVariables(context));
            context.setProperty(JXPATH_CONTEXT_PROPERTY, ctx);
        }   
        return ctx;     
    }
}
