/*
 * $Header: /home/cvs/jakarta-commons/jxpath/src/java/org/apache/commons/jxpath/util/TypeUtils.java,v 1.4 2002/04/26 03:28:37 dmitri Exp $
 * $Revision: 1.4 $
 * $Date: 2002/04/26 03:28:37 $
 *
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999-2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Commons", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, Plotnix, Inc,
 * <http://www.plotnix.com/>.
 * For more information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package com.plotnix.jex.util;

import java.lang.reflect.*;
import java.util.*;


/**
 * @author Dmitri Plotnikov
 * @version $Revision: 1.4 $ $Date: 2002/04/26 03:28:37 $
 */
public class TypeUtils {

    public static boolean canConvert(Object object, Class toType){
        if (object == null){
            return true;
        }

        if (toType == Object.class){
            return true;
        }

        Class fromType = object.getClass();
        if (fromType.equals(toType)){
            return true;
        }

        if (toType.isAssignableFrom(fromType)){
            return true;
        }

        if (toType == String.class){
            return true;
        }

        if (object instanceof Boolean){
            if (toType == boolean.class ||
                    Number.class.isAssignableFrom(toType)){
                return true;
            }
        }
        else if (object instanceof Number){
            if (toType.isPrimitive() ||
                    Number.class.isAssignableFrom(toType)){
                return true;
            }
        }
        else if (object instanceof Character){
            if (toType == char.class){
                return true;
            }
        }
        else if (object instanceof String){
            if (toType.isPrimitive()){
                return true;
            }
            if (toType == Boolean.class ||
                    toType == Character.class ||
                    toType == Byte.class ||
                    toType == Short.class ||
                    toType == Integer.class ||
                    toType == Long.class ||
                    toType == Float.class ||
                    toType == Double.class){
                return true;
            }
        }
        else if (fromType.isArray()){
            if (Array.getLength(object) == 1){
                Object value = Array.get(object, 0);
                return canConvert(value, toType);
            }
        }
        else if (object instanceof List){
            if (((List)object).size() == 1){
                Object value = ((List)object).get(0);
                return canConvert(value, toType);
            }
        }
        else if (object instanceof Collection){
            if (!((Collection)object).isEmpty()){
                Iterator it = ((Collection)object).iterator();
                Object value = it.next();
                return canConvert(value, toType);
            }
        }

        // TBD: date conversion to/from string
        return false;
    }

    public static Object convert(Object object, Class toType){
        if (object == null){
            return null;
        }

        if (toType == Object.class){
            return object;
        }


        Class fromType = object.getClass();
        if (fromType.equals(toType) || toType.isAssignableFrom(fromType)){
            return object;
        }

        if (toType == String.class){
            return object.toString();
        }

        if (object instanceof Boolean){
            if (toType == boolean.class){
                return object;
            }
            boolean value = ((Boolean)object).booleanValue();
            return allocateNumber(toType, value ? 1 : 0);
        }
        else if (object instanceof Number){
            double value = ((Number)object).doubleValue();
            if (toType == boolean.class || toType == Boolean.class){
                return value == 0.0 ? Boolean.FALSE : Boolean.TRUE;
            }
            if (toType.isPrimitive() ||
                    Number.class.isAssignableFrom(toType)){
                return allocateNumber(toType, value);
            }
        }
        else if (object instanceof Character){
            if (toType == char.class){
                return object;
            }
        }
        else if (object instanceof String){
            if (toType == boolean.class || toType == Boolean.class){
                return Boolean.valueOf((String)object);
            }
            if (toType == char.class || toType == Character.class){
                return new Character(((String)object).charAt(0));
            }
            if (toType == byte.class || toType == Byte.class){
                return new Byte((String)object);
            }
            if (toType == short.class || toType == Short.class){
                return new Short((String)object);
            }
            if (toType == int.class || toType == Integer.class){
                return new Integer((String)object);
            }
            if (toType == long.class || toType == Long.class){
                return new Long((String)object);
            }
            if (toType == float.class || toType == Float.class){
                return new Float((String)object);
            }
            if (toType == double.class || toType == Double.class){
                return new Double((String)object);
            }
        }
        else if (fromType.isArray()){
            Object value = Array.get(object, 0);
            return convert(value, toType);
        }
        else if (object instanceof List){
            Object value = ((List)object).get(0);
            return convert(value, toType);
        }
        else if (object instanceof Collection){
            Iterator it = ((Collection)object).iterator();
            Object value = it.next();
            return convert(value, toType);
        }
        return object;
    }

    private static Number allocateNumber(Class type, double value){
        if (type == Byte.class || type == byte.class){
            return new Byte((byte)value);
        }
        if (type == Short.class || type == short.class){
            return new Short((short)value);
        }
        if (type == Integer.class || type == int.class){
            return new Integer((int)value);
        }
        if (type == Long.class || type == long.class){
            return new Long((long)value);
        }
        if (type == Float.class || type == float.class){
            return new Float((float)value);
        }
        if (type == Double.class || type == double.class){
            return new Double(value);
        }
        return null;
    }
}