package com.plotnix.jex;

import junit.framework.TestCase;

import java.util.*;

/**
 * 
 *
 * @author Dmitri Plotnikov
 * @version $Revision$ $Date$
 */
public class JexTestCase extends TestCase {

    /**
     * Constructor for JexTestCase.
     * @param name of the test
     */
    public JexTestCase(String name) {
        super(name);
    }

    public static void main(String[] args) {
        junit.textui.TestRunner.run(JexTestCase.class);
    }

    public void testJxpathCompilerIntegration(){
        AccountBean bean = new AccountBean();
        
        Context context = new Context();
        Expression expr = context.getCompiler().
            compileExpression("jxpath:accountNumber");
            
        Object result = expr.evaluate(context, new Object[]{bean});
        assertEquals("jxpath:accountNumber", 
            AccountBean.ACCOUNT_NUMBER, result);
            
        context.getVariables().declareVariable("x", "2");        
        expr = context.getCompiler().
            compileExpression("jxpath:$x+2");
        result = expr.evaluate(context, null);
        assertEquals("jxpath:$x + 2", 
            new Double(4), result);        
    }

    public void testJxpathInterpreterIntegration(){
        AccountBean bean = new AccountBean();
        
        Context context = new Context();
        Object result = context.evaluate("jxpath:accountNumber", new Object[]{bean});
        assertEquals("jxpath:accountNumber", 
            AccountBean.ACCOUNT_NUMBER, result);
            
        result = context.evaluate("jxpath:transactions[1]", new Object[]{bean});
        assertEquals("jxpath:transactions[1]", "First", result);
        
        context.getVariables().declareVariable("x", "2");        
        result = context.evaluate("jxpath:$x+2", null);
        assertEquals("jxpath:$x + 2", 
            new Double(4), result);        

        List list = new ArrayList();
        Iterator it = context.iterate("jxpath:transactions", new Object[]{bean});
        while (it.hasNext()){
            list.add(it.next());
        }
        assertEquals("jxpath:transactions", 
            Arrays.asList(new String[]{"First", "Second"}), list);
    }

    public void testBexlCompilerIntegration(){
        AccountBean bean = new AccountBean();
        
        Context context = new Context();
        Expression expr = context.getCompiler().
            compileExpression("bexl:accountNumber");
            
        Object result = expr.evaluate(context, new Object[]{bean});
        assertEquals("bexl:accountNumber", 
            AccountBean.ACCOUNT_NUMBER, result);
    }

    public void testBexlInterpreterIntegration(){
        AccountBean bean = new AccountBean();
        
        Context context = new Context();
            
        Object result = context.evaluate("bexl:accountNumber", new Object[]{bean});
        assertEquals("bexl:accountNumber", 
            AccountBean.ACCOUNT_NUMBER, result);
            
        result = context.evaluate("bexl:transactions[0]", new Object[]{bean});
        assertEquals("bexl:transactions[0]", "First", result);
        
        List list = new ArrayList();
        Iterator it = context.iterate("bexl:transactions", new Object[]{bean});
        while (it.hasNext()){
            list.add(it.next());
        }
        assertEquals("bexl:transactions", 
            Arrays.asList(new String[]{"First", "Second"}), list);
    }    

    public void testDefaultLanguageInterpreter(){
        AccountBean bean = new AccountBean();
        
        Context context = new Context();
        context.setExpressionLanguage("bexl");
            
        Object result = context.evaluate("accountNumber", new Object[]{bean});
        assertEquals("accountNumber", 
            AccountBean.ACCOUNT_NUMBER, result);
    }    

    public void testDefaultLanguageCompiler(){
        AccountBean bean = new AccountBean();
        
        Context context = new Context();
        context.setExpressionLanguage("jxpath");
        
        Expression expr = context.getCompiler().
            compileExpression("accountNumber");
            
        Object result = expr.evaluate(context, new Object[]{bean});
        assertEquals("jxpath:accountNumber", 
            AccountBean.ACCOUNT_NUMBER, result);
    }

    public void testInterpreterSyntaxValidation(){
        Context context = new Context();
        boolean exception = false;
        try {
            // This is an invalid xpath expression
            context.getInterpreter().validateExpression("jxpath:...");
        }
        catch (Exception ex){
            exception = true;
        }
        assertEquals("Syntax checking exception", true, exception);
    }
    
    public void testJavaScriptIntegration(){
        AccountBean bean = new AccountBean();
        
        Context context = new Context();
        context.setExpressionLanguage("javascript");
        Expression expr = context.getCompiler().
            compileExpression("accountNumber");
            
        Object result = expr.evaluate(context, new Object[]{bean});
        assertEquals("accountNumber", 
            AccountBean.ACCOUNT_NUMBER, result);
            
        result = context.evaluate("transactions[0]", new Object[]{bean});
        assertEquals("transactions[0]", "First", result);
        
        context.getVariables().declareVariable("x", new Double(2));        
        result = context.evaluate("x+2", null);
        assertEquals("x + 2",  new Double(4), result);        
        
        context.setProperty("javascript.autodeclare", "true");
        context.assign("name", "John Smith", new Object[]{bean});
        result = context.evaluate("name", null);
        assertEquals("name",  "John Smith", result);
        
        List list = new ArrayList();
        Iterator it = context.iterate("transactions", new Object[]{bean});
        while (it.hasNext()){
            list.add(it.next());
        }
        assertEquals("transactions", 
            Arrays.asList(new String[]{"First", "Second"}), list);
    }

    public void testJavascriptSyntaxValidation(){
        Context context = new Context();
        boolean exception = false;
        try {
            // This is an invalid javascript expression
            context.getInterpreter().validateExpression("javascript:...");
        }
        catch (Exception ex){
            exception = true;
        }
        assertEquals("Syntax checking exception", true, exception);

        exception = false;
        try {
            // This is an invalid variable expression
            context.getInterpreter().
                    validateVariableExpression("javascript:2");
        }
        catch (Exception ex){
            exception = true;
        }
        assertEquals("Syntax checking exception", true, exception);
    }
    
}
